%MPM_ASbaffledradzmatrix : Modal radiation impedance matrix
%
% Zmat = MPM_ASbaffledradzmatrix(k, rho, c, S, maxmodes, bz)
% 
% Calculates the modal radiation impedance matrix for the 
% end of a circular tube terminated in an infinite baffle
% by numerical interation.
% The fundamental (plane wave) mode impedance is calcualted
% by analytical functions.
%
% The matrix Zmat is a square, symmetrical matrix. 
%
% Input parameters:
% k : wavenumber
% rho : density of medium
% c : sound speed in medium
% S : area of tube opening
% maxmodes : maximum number of modes calculated
% bz : zeros of Bessel function J1
%
%  --------------------------------------------------- -------------------------------------------
%    This file is part of the Modal Propagation Method (MPM) Toolbox by Bjrn Kolbrek.                       
%                                                                                               
%    The MPM Toolbox is free software: you can redistribute it and/or modify       
%    it under the terms of the GNU General Public License as published by the Free Software     
%    Foundation, either version 3 of the License, or (at your option) any later version.        
%                                                                                                
%    The MPM Toolbox is distributed in the hope that it will be useful,       
%    but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS  
%    FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.             
%                                                                                                
%    You should have received a copy of the GNU General Public License along with the           
%    MPM Toolbox. If not, see <http://www.gnu.org/licenses/>.                 
%  --------------------------------------------------- -------------------------------------------
%

function Zmat = MPM_ASbaffledradzmatrix(k, rho, c, S, maxmodes, bz)
Zmat = zeros(maxmodes, maxmodes, length(k));
kR = k*sqrt(S/pi);
eps = 10.^(-log(kR)-1);
eps = max(min(0.01,eps),1e-6);
if length(k) > 1 
    eps = 1e-6;
end;
for m=1:maxmodes
    for n=1:maxmodes
        %disp (['for mode m,n = ' num2str(m) ',' num2str(n)]);
        if (n==1)&&(m==1)
            % fundamental mode: use analytical solution
            R = 1-besselj(1,2*kR)./(kR);
            X = 2*kR.*struveH1(2*kR)./(2*kR.^2);
            Z = R+1i*X;
            %disp (['calc m,n = ' num2str(m) ',' num2str(n)]);
        elseif n>=m
            R = quadv(@(x) MPM_ASbresistance(x,n,m,kR,bz), 0,pi/2, eps);
            X = quadv(@(x) MPM_ASbreactance(x,n,m,kR,bz), 0,10, 1e-6);
            Z = R+1i*X;
            %disp (['calc m,n = ' num2str(m) ',' num2str(n)]);
        else
            Z = Zmat(n,m,:);
            %disp (['copy m,n = ' num2str(m) ',' num2str(n) ' = ' num2str(n) ',' num2str(m)]);
        end;
        Zmat(m,n,:) = Z;
    end;
end;

Zmat = rho*c/S*Zmat;

function dR = MPM_ASbresistance(phi,n,m,kR,bz)
sinphi = sin(phi);
dR = sinphi.*FuncDn(sinphi, bz(n), kR).*FuncDn(sinphi, bz(m), kR);

function dX = MPM_ASbreactance(phi,n,m,kR,bz)
coshphi = cosh(phi);
dX = coshphi.*FuncDn(coshphi, bz(n), kR).*FuncDn(coshphi, bz(m), kR);

function D = FuncDn(tau, gamma_n, kR)
D = -sqrt(2).*tau.*besselj(1,(tau.*kR))./ ((gamma_n./kR).^2-tau.^2);