%MPM_ASgetDI
%
%  DI = MPM_ASgetDI(k,angles, pext)
%
%  Calculates the directivity index from an array of field point pressures.
%  DI is calculated from Beranek, eq. 4.24 and 4.19.
%  If not enough field points are provided, the integration is performed
%  using weights calculated from Gerzon: "Calculating the Directivity Factor
%  of Transducers from Limited Polar Diagram Information", JAES 1975, p.
%  369
%
%  Input parameters:
%  k : wavenumber, assumed to be a row vector
%  angles : angles in degrees, assumed to be a column vector
%  pext : field point pressures, na rows by nk columns
%
%  --------------------------------------------------- -------------------------------------------
%    This file is part of the Modal Propagation Method (MPM) Toolbox by Bjrn Kolbrek.                       
%                                                                                               
%    The MPM Toolbox is free software: you can redistribute it and/or modify       
%    it under the terms of the GNU General Public License as published by the Free Software     
%    Foundation, either version 3 of the License, or (at your option) any later version.        
%                                                                                                
%    The MPM Toolbox is distributed in the hope that it will be useful,       
%    but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS  
%    FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.             
%                                                                                                
%    You should have received a copy of the GNU General Public License along with the           
%    MPM Toolbox. If not, see <http://www.gnu.org/licenses/>.                 
%  --------------------------------------------------- -------------------------------------------
%
%  see also Hints, MPM_Horndemo2, MPM_AShorncoord, MPM_refinecoords, MPM_makesteps,
%  MPM_ASbaffledradzmatrix, MPM_getimpedances, MPM_makebigfmat,
%  MPM_getmouthvolvel, MPM_getfieldpointpressures
%  
function DI = MPM_ASgetDI(k,angles, pext)

if length(angles)>100
    dtheta = pi/180*angles(2);
    sia = dtheta*sin(pi/180*angles);
    sia = sia(:,ones(1,length(k)));
    Wrad = sum((pext).^2.*sia);
    Q = 2*(pext(1,:).^2)./Wrad;
else    
    dang = angles(2);
    n = round(180/dang);

    m = n/2 + 1;
    wt = zeros(m,1);
    % calculate weights for numerical integration (after Gerzon 1975)
    for r=0:2:n
        if (r==0)|(r==n)
            k1 = 0.5;
        else
            k1 = 1;
        end;
        wt(1) = wt(1) + k1*(-1/(r^2-1));
    end;
    wt(1) = wt(1)/(n);

    for i=1:m-1
        wt(i+1) = 0;
        for r=0:2:n
            if (r==0)|(r==n)
                k1 = 0.5;
            else
                k1 = 1;
            end;
            wt(i+1) = wt(i+1)+ k1*(-1/(r^2-1)) * cos(pi*r*i/n);
            %disp(num2str([r i cos(pi*r*i/n)]))
        end;
         wt(i+1) = 2*wt(i+1)/n;
    end;
    s = size(pext);
    wt = wt(:,ones(1,s(2)));
    Q = sum(wt.*pext.^2);
    Q = pext(1,:).^2./Q;

    maxq = (0.5*n+1)^2*sqrt(2);
    if max(Q) > maxq
        warning(sprintf('Directivity index larger than %2.0fdB is unreliable. \nPlease use more field points.',10*log10(maxq)));
    end;    
end;
DI = 10*log10(abs(Q));