%function U = MPM_ASgetvolvelmodes(k, u0, a, N, shape, R, bz)
%
% U = MPM_ASgetvolvelmodes(k, u0, a, N, shape, R, bz)
%
% Calculates the modal volume velocity amplitudes for a wavefront of a
% given shape.
%
% Input parameters:
% k : wavenumber
% a : aperture radius
% U0 : average volume velocity normal to the wave front
% N : number of modes
% shape : (optional) sets the wave front shape. 
%   'planar' sets a plane wave front
%   'spherical' sets a spherical wave front with radius R
% R : (optional of shape = planar) radius of spherical wave front
% bz : zeros of Bessel function J1
%
%  --------------------------------------------------- -------------------------------------------
%    This file is part of the Modal Propagation Method (MPM) Toolbox by Bjrn Kolbrek.                       
%                                                                                               
%    The MPM Toolbox is free software: you can redistribute it and/or modify       
%    it under the terms of the GNU General Public License as published by the Free Software     
%    Foundation, either version 3 of the License, or (at your option) any later version.        
%                                                                                                
%    The MPM Toolbox is distributed in the hope that it will be useful,       
%    but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS  
%    FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.             
%                                                                                                
%    You should have received a copy of the GNU General Public License along with the           
%    MPM Toolbox. If not, see <http://www.gnu.org/licenses/>.                 
%  --------------------------------------------------- -------------------------------------------
%

function U = MPM_ASgetvolvelmodes(k, U0, N, a, shape, R, bz)
if nargin < 4
    shape = 'planar';
end;
if nargin < 2 
    N = 1;
    shape = 'planar';
end;
if (nargin < 7)&(strcmpi(shape, 'spherical') == 1)
    warning('Error in MPM_ASgetvolvelmodes : wave front radius must be set! Using plane wave front instead.');
    shape = 'planar';
end;

Nk = length(k);

if strcmpi(shape, 'planar') == 1
    U = zeros(N,Nk);
    if length(U0) ~= length(k)
        U0 = U0(1)* ones(1,Nk);
    end;
    U(1,:) = U0;
end;

if strcmpi(shape, 'spherical') == 1 
    % radial vector with as many components as there is modes
    r = linspace(0,a,N);
    phi = MPM_ASgeteigenfunctions(a, r,  bz(1:N), true);
    sk = size(k);
    Nk = sk(2);
    r = r';
    l = sqrt(R^2-a^2);
    lpx = (l^2 + r.^2);
    lpx = lpx(:,ones(1,Nk));
    km = k(ones(N,1), :);
    % velocity
    v = (l^2./lpx .* exp(-1i.*km.*(sqrt(lpx)-l)));
    % velocity modal amplitudes
    V = phi\v;
    if length(U0) ~= length(k)
        U0 = U0(1)* ones(1,Nk);
    end;
    % multiply by the average amplitudes for each k
    U0 = U0(ones(N,1),:);    
    U = V .* U0;    
end

% if strcmpi(shape, 'spherical2') == 1    
%     % this method doesn't work as well
%     U = zeros(N,Nk);
%     if length(U0) ~= length(k)
%         U0 = U0(1)* ones(1,Nk);
%     end;
%     l = sqrt(R^2-a^2);
%     % find the modal coefficients, from Geddes, eq. 4.1.11 and 4.2.14
%     for ii = 1:N
%         U(ii,:) = U0.* quadv(@(x)sphericalaperturefunc(x, k, a, l, bz,ii), 0,a,1e-8);
%     end;
% end;

function y = sphericalaperturefunc(x, k, a, l, bz,ii)
% from Geddes, eq. 4.2.13
barg = bz(ii)*x/a;
lpx = (l^2 + x.^2);
v = l^2./lpx .* exp(1i.*k.*(sqrt(lpx)-l));%ones(size(k));%
y = v.*besselj(0,barg).*x;
    
    