%MPM_makefmat : Make scattering matrix F
% 
% F = MPM_ASmakefmat(N,R1,R2,bz);
% 
% Calculates the scattering matrix F, used to propagate modes across
% a discontinuity (axisymmetric case).
% 
% Returns the inverse of the matrix V if R1>R2
% 
% Input parameters: 
% N : number of modes
% R1 : radius of tube 1
% R2 : radius of tube 2
% bz : zeros of Bessel function J1
% forceexact : set to true to avoid approximation for small R1/R2
%
%  --------------------------------------------------- -------------------------------------------
%    This file is part of the Modal Propagation Method (MPM) Toolbox by Bjrn Kolbrek.                       
%                                                                                               
%    The MPM Toolbox is free software: you can redistribute it and/or modify       
%    it under the terms of the GNU General Public License as published by the Free Software     
%    Foundation, either version 3 of the License, or (at your option) any later version.        
%                                                                                                
%    The MPM Toolbox is distributed in the hope that it will be useful,       
%    but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS  
%    FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.             
%                                                                                                
%    You should have received a copy of the GNU General Public License along with the           
%    MPM Toolbox. If not, see <http://www.gnu.org/licenses/>.                 
%  --------------------------------------------------- -------------------------------------------
%

function F = MPM_ASmakefmat(N,R1,R2,bz, forceexact, forceF);
if nargin<5 
    forceexact = false;
end;
if nargin < 6
    forceF = false;
end;

beta = R1/R2;
vmatrix = false;

if (beta > 1)&(~forceF)
    beta = 1/beta;
elseif (beta==1)
    F = eye(N);
    return;
end;

F = zeros(N);
    
if (beta > 0.9995)&(forceexact==false) %small cross sectional change, error of approximation will be less than 1e-4 
    Q = F;
    S1 = pi*R1^2;
    S2 = pi*R2^2;
    epsilon = (S2-S1)/S1;
    if (epsilon > 1e-9)
        gamma_m =bz(:,ones(1,N)).';
        gamma_n =bz(:,ones(1,N));
        Q = gamma_m.^2./(gamma_m.^2-gamma_n.^2);
        Qd = eye(N);
        di = find(Qd>0);
        Q(di)=0;
        F = eye(N) - epsilon*Q;
    else
        F = eye(N);
    end;
else  
    gamma_m =bz(:,ones(1,N)).';
    gamma_n =bz(:,ones(1,N));
    Fm = 2*beta*gamma_m.*besselj(1,beta*gamma_m)./ besselj(0,gamma_m);
    F = Fm./(beta^2*gamma_m.^2 - gamma_n.^2);
end;
F(1,1) = 1;

