%MPM_ASrayleighint : Rayleigh integral
%
% prext = MPM_rayleighint(k, vert, vvel, pe, rho, c)
% 
% Calculates the radiated pressure in front of a baffled 
% radiator with a given velocity distribution, by the 
% Rayleigh integral. 
% The radiator is assumed to be in the z = 0 plane.
%
% Input parameters:
% k : wavenumber
% vert : vertices for the radiator. Coordinates for the r values.
%        The integration is performed along the midpoint of the rings 
%        defined by these points.
% vvel : particle velocity at the midpoint of the rings
% pe : exterior point, (r,z). 
% rho : density of the medium
% c : sound speed in medium
%
%  --------------------------------------------------- -------------------------------------------
%    This file is part of the Modal Propagation Method (MPM) Toolbox by Bjrn Kolbrek.                       
%                                                                                               
%    The MPM Toolbox is free software: you can redistribute it and/or modify       
%    it under the terms of the GNU General Public License as published by the Free Software     
%    Foundation, either version 3 of the License, or (at your option) any later version.        
%                                                                                                
%    The MPM Toolbox is distributed in the hope that it will be useful,       
%    but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS  
%    FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.             
%                                                                                                
%    You should have received a copy of the GNU General Public License along with the           
%    MPM Toolbox. If not, see <http://www.gnu.org/licenses/>.                 
%  --------------------------------------------------- -------------------------------------------
%#codegen

function prext = MPM_ASrayleighint(k, vert, vvel, pe, rho, c)

prext = 0+0i;
phiext = 0+0i;
p = [pe(1) 0 pe(2)];
NV = length(vert);
S = 0;

for ir=1:NV-1
    QA=vert(ir);
    QB=vert(ir+1);
    radmid = (QA+QB)/2;
    glen = abs(QA-QB);
    cirmid = 2*pi*radmid;
    % number of points in theta direction proportional to 
    % size factor (circumference/radius)
    NT = ceil(1+cirmid/glen);
    dS = pi * (QB^2-QA^2);
    for it=1:NT
        theta = (it-1)*2*pi/NT;
        dtheta = 2*pi/NT;
        dS = 0.5*dtheta*(QB^2-QA^2);
        q = p;
        q(1) = radmid*cos(theta);
        q(2) = radmid*sin(theta);
        q(3) = 0;
        S = S+dS;
        r = norm(p-q);
        phiext = phiext + (vvel(ir,:).*dS.*exp(-1i*k*r) / r);
    end;    
end;

prext = 1i*k*rho*c./(2*pi).*phiext;

