%MPM_Hornsystemdemo1
%
%  This file contains sample code showing how the MPM toolbox can be used to
%  calculate the performance of a complete horn loudspeaker. Given the
%  parameters of a driver, and rear and front chambers, the frequency
%  response in the far field can be calculated. 
%
%  A plane wave is assumed at the throat/throat chamber input. The mouth
%  volume velocity is dictated both by the horn throat impedance, the rear
%  chamber, and the driver parameters.
%
%  For comparison, the driver parameters are the same as for the Hornresp
%  default horn speaker, but the horn itself has been changed to have one
%  cutoff wavelength mouth circumference for half-space radiation
%  conditions. 
%
%  Several horn types can be specified, see MPM_AShorncoord for details.
%  
%  See Hints.m for hints on usage.
%
%  --------------------------------------------------- -------------------------------------------
%    This file is part of the Modal Propagation Method (MPM) Toolbox by Bjrn Kolbrek.                       
%                                                                                               
%    The MPM Toolbox is free software: you can redistribute it and/or modify       
%    it under the terms of the GNU General Public License as published by the Free Software     
%    Foundation, either version 3 of the License, or (at your option) any later version.        
%                                                                                                
%    The MPM Toolbox is distributed in the hope that it will be useful,       
%    but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS  
%    FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.             
%                                                                                                
%    You should have received a copy of the GNU General Public License along with the           
%    MPM Toolbox. If not, see <http://www.gnu.org/licenses/>.                 
%  --------------------------------------------------- -------------------------------------------
%
%  see also Hints, MPM_Horndemo1, MPM_AShorncoord, MPM_refinecoords, MPM_makesteps,
%  MPM_ASbaffledradzmatrix, MPM_getimpedances, MPM_makebigfmat,
%  MPM_getmouthvolvel, MPM_getfieldpointpressures
%  

%clear all;
% load the precalculated zeros of Bessel function J1
load 'MPM_besselzeros.mat';
 
% horn parameters
sth = 80e-4; % throat area
smh = 9609e-4; % mouth area;
Lh = 187e-2; % horn length
dz = Lh/300; % segment length
Tn = 0.7; % horn parameter (hypex/bessel)
HornType = 1; 
AddRadius = false; % set to true to gat a radiused flaring of the mouth
RadR = 0.2; % radius of mouth radius
RFta = 90; % end tangent angle of radius (degrees)

% Driver parameters
Sd = 350e-4; %m^2
Re = 6.0; % ohms
Le = 1.0e-3; % H
BL = 18; % Tm
Cms = 4.0e-4; % m/N
Rms = 4.0; %Ns/m
Mmd = 20.0e-3; %kg

% Front and rear chamber parameters
Vrc = 14e-3; %m^3
Lrc = 16e-2; %m
Vtc = 0*900e-6; %m^3
Atc = 350e-4; % m^2

% Generator parameters
Eg = 2.83;
Rg = 0;

% field point parameters
% field point distance
Rext = 3;
% field point angles (degrees)
Angext = linspace(0,90,10)';


% simulation parameters
fmin = 10; %lower frequency
fmax = 5000; % upper frequency
Nf = 150; % number of frequencies
N = 4; %maximum number of modes
c = 344; %sound speed
rho = 1.205;% air density

% ----------------------------------------------------------------------
%  Calculations 

AddVtc = ((Atc*Vtc)>0);

% Create horn contour
horncoords = MPM_AShorncoord(HornType, sth, smh, Lh, Tn, dz, AddRadius, 0, RadR, RFta);

if AddVtc
    Rvtc = sqrt(Atc/pi);
    Lvtc = Vtc/Atc;
    horncoords(:,1) = horncoords(:,1)+Lvtc;
    horncoords = [0 Rvtc;
        Lvtc Rvtc;
        horncoords];
    newdz = dz/10;
    horncoords = MPM_refinecoords(horncoords, dz);
end;
horncoords = MPM_makesteps(horncoords);
 
% plot horn contour
figure(1);
plot(horncoords(:,1),horncoords(:,2))
ylim([0,max(horncoords(:,2)*1.1)])
xlabel('z axis [m]');
ylabel('Radius [m]');
title('Horn profile');

freq = logspace(log10(fmin), log10(fmax), Nf);
k = 2*pi*freq/c;
w = 2*pi*freq;
% alternative method for calculating bz.
% function besselzero can be downloaded from
% the Matlab Central.
%bz = zeros(N,1);
%if (N>1)
%    bz(2:N)= besselzero(1, N-1, 1);
%end;
bz = bz(1:N);

% Calculate driver acoustical source impedance

Zee = Rg+Re+1i*w*Le;
Zme = BL^2./Zee;
Zm = Rms + 1i*w*Mmd + 1./(1i*w*Cms);
Zmt = Zme + Zm;
Zas = Zmt*1/Sd^2;
Ps = Eg*BL/Sd./Zee;
if (Lrc*Vrc)>0
    Zab = -1i*rho*c/(Vrc/Lrc)*cot(k*Lrc);
else
    Zab = 0;
end;

%% -----------------------------------------------
%  The performance calculations

t00 = tic; 
BigF = MPM_makebigfmat(N, horncoords, bz);

disp(['Calculating radiation impedance']);
Sm = pi*horncoords(end,2)^2;
St = pi*horncoords(1,2)^2;
Zrad = MPM_ASbaffledradzmatrix(k,rho,c,Sm,N,bz);
toc(t00);

disp(['Calculating horn impedance']);
BigZ = MPM_getimpedances(k, horncoords, bz, Zrad, BigF, rho, c, 0, true);
toc(t00);
Z00mat = squeeze(BigZ(1,1,1,:));
Z00matd = St/(rho*c)*Z00mat;
%%
figure(2);
semilogx(freq, real(Z00matd), 'k', freq, imag(Z00matd), 'r');
grid;
title('Horn throat impedance');
ylabel(['Normalized acoustic Z(' num2str(0) ',' num2str(0) ')']);
xlabel('Hz');

%%
U0 = zeros(N,Nf); %plane wave at throat
U0(1,:) = (Ps./(Zas+Zab+Z00mat.')).';

Nr = 50;
pe = [Rext*sin(Angext/180*pi) Rext*cos(Angext/180*pi)];
%%
disp(['Calculating mouth volume velocity']);
Umouth = MPM_getmouthvolvel(k, horncoords, U0, bz, BigZ, BigF, rho, c, 0);
toc(t00);
%%
disp(['Calculating field point pressure']);
pext = MPM_getfieldpointpressures(k, Umouth, pe, horncoords(end,2), 25, bz, rho, c, 0);
toc(t00);
%%
figure(3);
semilogx(freq, 94+20*log10(abs(pext)));
%hold off;
ylim([60 110]);
ylabel('dB re 20Pa');
xlabel('Hz');
legend(num2str(Angext), 'Location', 'NorthWest')
title('Field point pressures');
grid;

%%
figure(4);
contourf(freq, Angext, 94+20*log10(abs(pext)), 15);
set(gca, 'xscale', 'log');
ylabel('Degrees');
xlabel('Hz');
title('Polar map');
colorbar;

%placeFigures;
