%MPM_Horndemo2
%
%  This file contains sample code showing how the MPM toolbox can be used to
%  calculate the performance of a horn exited by a plane wave at the throat.
%  The horn contour is first calculated, and cast into the form of a stepped
%  duct. Next, the modal radiation impedance at the mouth is calculated.
%  Following this, the throat impedance, and the modal impedances throughout
%  the horn are calculated. This data is then used in the propagation of
%  velocity from throat to mouth. The mouth velocity is used to compute the
%  resulting pressure in the free field, using the Rayleigh integral. 
%
%  Several horn types can be specified, see MPM_AShorncoord for details.
%
%  This file is similar to MPM_Horndemo1, but shows how to compute a
%  two-segment horn with throat chamber and mouth radius flare.
%  Note the dips in the response due to standing waves in the throat chamber.
%
%  See Hints.m for hints on usage.
%
%  --------------------------------------------------- -------------------------------------------
%    This file is part of the Modal Propagation Method (MPM) Toolbox by Bjrn Kolbrek.                       
%                                                                                               
%    The MPM Toolbox is free software: you can redistribute it and/or modify       
%    it under the terms of the GNU General Public License as published by the Free Software     
%    Foundation, either version 3 of the License, or (at your option) any later version.        
%                                                                                                
%    The MPM Toolbox is distributed in the hope that it will be useful,       
%    but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS  
%    FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.             
%                                                                                                
%    You should have received a copy of the GNU General Public License along with the           
%    MPM Toolbox. If not, see <http://www.gnu.org/licenses/>.                 
%  --------------------------------------------------- -------------------------------------------
%
%  see also hints, MPM_Horndemo1, MPM_AShorncoord, MPM_refinecoords, MPM_makesteps,
%  MPM_ASbaffledradzmatrix, MPM_getimpedances, MPM_makebigfmat,
%  MPM_getmouthvolvel, MPM_getfieldpointpressures

%clear all;
% load the precalculated zeros of Bessel function J1
load 'MPM_besselzeros.mat';
 
% horn parameters
sth1 = 10e-4; % throat area segment 1
smh1 = 100e-4; % mouth area segment 1
Lh1 = 10e-2; % horn length segment 1
Tn1 = 0.7; % horn parameter (hypex/bessel);
HornType1 = 0; 
smh2 = 110e-4; % mouth area segment 2
Lh2 = 10e-2; % horn length segment 2
Tn2 = 10;% horn parameter (hypex/bessel);
HornType2 = 0; 

dz = 0.0003; % segment length
AddRadius = false; % set to true to gat a radiused flaring of the mouth
RadR = 0.05; % radius of mouth radius
RFta = 90; % end tangent angle of radius (degrees)

% throat chamber
Rvtc = 0.2; %throat chamber radius
Lvtc = 0.02; % throat chamber length
AddVtc = false; % set to true to use throat chamber

% field point parameters
% field point distance
Rext = 3;
% field point angles (degrees)
Angext = linspace(0,90,10)';


% simulation parameters
fmin = 50; %lower frequency
fmax = 5000; % upper frequency
Nf = 100; % number of frequencies
N = 4; %maximum number of modes
c = 344; %sound speed
rho = 1.205;% air density
normalized = true; % normalize polar map to on-axis response

% Create horn contour
horncoords1 = MPM_AShorncoord(HornType1, sth1, smh1, Lh1, Tn1, dz);
horncoords2 = MPM_AShorncoord(HornType2, smh1, smh2, Lh2, Tn2, dz, AddRadius, 0, RadR, RFta);
horncoords2(:,1) = horncoords2(:,1) + Lh1;

horncoords = [horncoords1; horncoords2];

if AddVtc
    horncoords(:,1) = horncoords(:,1)+Lvtc; %+3*dz;
    horncoords = [0 Rvtc;
        Lvtc Rvtc;
%         Lvtc+3*dz horncoords(1,2);
        horncoords];
    newdz = dz/10;
    horncoords = MPM_refinecoords(horncoords, dz);
end;
 horncoords = MPM_makesteps(horncoords);
 
% plot horn contour
figure(1);
plot(horncoords(:,1),horncoords(:,2))
ylim([0,max(horncoords(:,2)*1.1)])
xlabel('z axis [m]');
ylabel('Radius [m]');
title('Horn profile');

freq = logspace(log10(fmin), log10(fmax), Nf);
k = 2*pi*freq/c;
% alternative method for calculating bz.
% function besselzero can be downloaded from
% the Matlab Central.
%bz = zeros(N,1);
%if (N>1)
%    bz(2:N)= besselzero(1, N-1, 1);
%end;
bz = bz(1:N);

%% -----------------------------------------------
%  The performance calculations

t00 = tic; 
BigF = MPM_makebigfmat(N, horncoords, bz);

disp(['Calculating radiation impedance']);
Sm = pi*horncoords(end,2)^2;
St = pi*horncoords(1,2)^2;
Zrad = MPM_ASbaffledradzmatrix(k,rho,c,Sm,N,bz);
toc(t00);

disp(['Calculating horn impedance']);
BigZ = MPM_getimpedances(k, horncoords, bz, Zrad, BigF, rho, c, 0, true);
toc(t00);
Z00mat = squeeze(BigZ(1,1,1,:));
Z00mat = St/(rho*c)*Z00mat;

figure(2);
semilogx(freq, real(Z00mat), 'k', freq, imag(Z00mat), 'r');
grid;
title('Horn throat impedance');
ylabel(['Normalized acoustic Z(' num2str(0) ',' num2str(0) ')']);
xlabel('Hz');

%%
U0 = zeros(N,1); %plane wave at throat
U0(1) = 1*St; %unit velocity at throat (not volume velocity)
Nr = 50;
pe = [Rext*sin(Angext/180*pi) Rext*cos(Angext/180*pi)];

disp(['Calculating mouth volume velocity']);
Umouth = MPM_getmouthvolvel(k, horncoords, U0, bz, BigZ, BigF, rho, c, 0);
toc(t00);

%%
disp(['Calculating field point pressure']);
pext = MPM_getfieldpointpressures(k, Umouth, pe, horncoords(end,2), 50, bz, rho, c, 0);
toc(t00);
%%
disp(['Calculating field point pressure modal method']);
pext2 = pext;
vm = Umouth/Sm;
for ii = 1: length(pe)
    pext2(ii,:) = MPM_ASmodalradiatedpressure(k, horncoords(end,2), vm,  pe(ii,:), bz, rho, c);
end;    
%%
% calculate normalized response
pext0 = pext(1,:);
pext0 = pext0(ones(length(pe),1),:);
pextn = pext ./ pext0;

figure(4);
semilogx(freq, 94+20*log10(abs(pext2)));
%hold off;
ylabel('dB re 20uPa');
xlabel('Hz');
legend(num2str(Angext), 'Location', 'NorthWest')
title('Field point pressures');

%%
figure(4);
if normalized 
    contourf(freq, Angext, 94+20*log10(abs(pextn)), 15);
    title('Polar map (normalized to on-axis response)');
else
    contourf(freq, Angext, 94+20*log10(abs(pext)), 15);
    title('Polar map');
end;
set(gca, 'xscale', 'log');
ylabel('Degrees');
xlabel('Hz');
colorbar;

disp(['Calculated horn with ' num2str(length(horncoords)/2) ' elements, using ' num2str(N) ' modes, at ' num2str(Nf) ' frequencies']);
placeFigures;
