%MPM_Horndemo3
%
%  This file contains sample code showing how the MPM toolbox can be used to
%  calculate the performance of a horn exited by a plane wave at the throat.
%  The horn contour is first calculated, and cast into the form of a stepped
%  duct. Next, the modal radiation impedance at the mouth is calculated by 
%  interpolation to speed up the calculations.
%  Following this, the throat impedance, and the modal impedances throughout
%  the horn are calculated. This data is then used in the propagation of
%  velocity from throat to mouth. The mouth velocity is used to compute the
%  resulting pressure in the free field, using a far field modal approach. 
%
%  Several horn types can be specified, see MPM_AShorncoord for details.
%  
%  See Hints.m for hints on usage.
%
%  --------------------------------------------------- -------------------------------------------
%    This file is part of the Modal Propagation Method (MPM) Toolbox by Bjrn Kolbrek.                       
%                                                                                               
%    The MPM Toolbox is free software: you can redistribute it and/or modify       
%    it under the terms of the GNU General Public License as published by the Free Software     
%    Foundation, either version 3 of the License, or (at your option) any later version.        
%                                                                                                
%    The MPM Toolbox is distributed in the hope that it will be useful,       
%    but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS  
%    FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.             
%                                                                                                
%    You should have received a copy of the GNU General Public License along with the           
%    MPM Toolbox. If not, see <http://www.gnu.org/licenses/>.                 
%  --------------------------------------------------- -------------------------------------------
%
%  see also Hints, MPM_Horndemo1, MPM_AShorncoord, MPM_refinecoords, MPM_makesteps,
%  MPM_ASbaffledradzmatrix, MPM_getimpedances, MPM_makebigfmat,
%  MPM_getmouthvolvel, MPM_getfieldpointpressures
%  

%clear all;
% load the precalculated zeros of Bessel function J1
load 'MPM_besselzeros.mat';
 
% horn parameters
sth = 10e-4; % throat area
smh = 500e-4; % mouth area;
Lh = 30e-2; % horn length
dz = Lh/1000; %0.0003; % segment length
Tn = 1; % horn parameter (hypex/bessel)
HornType = 1; 
AddRadius = false; % set to true to gat a radiused flaring of the mouth
RadR = 0.1; % radius of mouth radius
RFta = 90; % end tangent angle of radius (degrees)


Rvtc = 0.05;
Lvtc = 0.02;
AddVtc = false;

% field point parameters
% field point distance
Rext = 3;
% field point angles (degrees)
Angext = linspace(0,90,91)';


% simulation parameters
fmin = 100; %lower frequency
fmax = 10000; % upper frequency
Nf = 200; % number of frequencies
N = 16; %maximum number of modes
c = 344; %sound speed
rho = 1.205;% air density

% Create horn contour
horncoords = MPM_AShorncoord(HornType, sth, smh, Lh, Tn, dz, AddRadius, 0, RadR, RFta);

if AddVtc
    horncoords(:,1) = horncoords(:,1)+Lvtc;
    horncoords = [0 Rvtc;
        Lvtc Rvtc;
        horncoords];
    newdz = dz/10;
    horncoords = MPM_refinecoords(horncoords, dz);
end;
horncoords = MPM_makesteps(horncoords);
 
% plot horn contour
figure(1);
plot(horncoords(:,1),horncoords(:,2))
ylim([0,max(horncoords(:,2)*1.1)])
xlabel('z axis [m]');
ylabel('Radius [m]');
title('Horn profile');

freq = logspace(log10(fmin), log10(fmax), Nf);
k = 2*pi*freq/c;
% alternative method for calculating bz.
% function besselzero can be downloaded from
% the Matlab Central.
%bz = zeros(N,1);
%if (N>1)
%    bz(2:N)= besselzero(1, N-1, 1);
%end;
bz = bz(1:N);

%% -----------------------------------------------
%  The performance calculations

t00 = tic; 
BigF = MPM_makebigfmat(N, horncoords, bz);

disp(['Calculating radiation impedance']);
Sm = pi*horncoords(end,2)^2;
St = pi*horncoords(1,2)^2;
%Sm = Sm/2;
Zrad = MPM_ASbaffledradzmatrixIntp(k,rho,c,Sm,N);

toc(t00);
%%
disp(['Calculating horn impedance']);
BigZ = MPM_getimpedances(k, horncoords, bz, Zrad, BigF, rho, c, 0, false);
toc(t00);
Z00mat = squeeze(BigZ(1,1,1,:));
Z00mat = St/(rho*c)*Z00mat;


%%
figure(2);
semilogx(freq, real(Z00mat), 'k', freq, imag(Z00mat), 'r');
grid;
title('Horn throat impedance');
ylabel(['Normalized acoustic Z(' num2str(0) ',' num2str(0) ')']);
xlabel('Hz');


%%
U0 = zeros(N,1); %plane wave at throat
U0(1) = 1*St; %unit velocity at throat (not volume velocity)
Nr = 50;

disp(['Calculating mouth volume velocity']);
Umouth = MPM_getmouthvolvel(k, horncoords, U0, bz, BigZ, BigF, rho, c, 0);
toc(t00);
%%

pe = [Rext*sin(Angext/180*pi) Rext*cos(Angext/180*pi)];
disp(['Calculating field point pressure']);
% Rayleigh integral
% pext = MPM_getfieldpointpressures(k, Umouth, pe, horncoords(end,2), 50, bz, rho, c, 0);
% Modal method (faster, for the far-field)
pext = MPM_ASmodalradiatedpressure(k, sqrt(Sm/pi), Umouth, pe, bz, rho, c);
toc(t00);
%%
figure(3);
ia = find(mod(Angext, 10)==0);
semilogx(freq, 94+20*log10(abs(pext(ia,:))));
%hold off;
ylabel('dB re 20uPa');
xlabel('Hz');
%legend(num2str(Angext(ia)), 'Location', 'NorthWest')
title('Field point pressures');


figure(4);
contourf(freq, Angext, 94+20*log10(abs(pext)), 15);
set(gca, 'xscale', 'log');
ylabel('Degrees');
xlabel('Hz');
title('Polar map');
colorbar;

% calculate DI
DI = MPM_ASgetDI(k, Angext, pext);

figure(5);
semilogx(freq, DI);
xlabel('Hz');
ylabel('dB');
title('Directivity index');

placeFigures;
