%MPM_RECAmakefmat : Make scattering matrix F
% 
% F = MPM_RECAmakefmat(N,a1,a2,b1,b2, forceexact);
% 
% Calculates the scattering matrix F, used to propagate modes across
% a discontinuity (rectangular, asymmetric case).
% 
% Input parameters: 
% N : number of modes in each direction
% a1 : x-widths of tube 1
% a2 : x-widths of tube 2
% b1 : y-heights of tube 1
% b2 : y-heights of tube 2
%  a1(1) is the negative (left) width, a(2) is the positive (right) width
%  etc.
% forceexact : set to true to avoid approximation for small a1/a2 (not
% implemented)
%
%  --------------------------------------------------- -------------------------------------------
%    This file is part of the Modal Propagation Method (MPM) Toolbox by Bjrn Kolbrek.                       
%                                                                                               
%    The MPM Toolbox is free software: you can redistribute it and/or modify       
%    it under the terms of the GNU General Public License as published by the Free Software     
%    Foundation, either version 3 of the License, or (at your option) any later version.        
%                                                                                                
%    The MPM Toolbox is distributed in the hope that it will be useful,       
%    but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS  
%    FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.             
%                                                                                                
%    You should have received a copy of the GNU General Public License along with the           
%    MPM Toolbox. If not, see <http://www.gnu.org/licenses/>.                 
%  --------------------------------------------------- -------------------------------------------
%

function F = MPM_RECAmakefmat(N,a1,a2,b1,b2, forceexact);
if nargin<6 
    forceexact = false;
end;

alphax1 = a1(2)-a1(1);
alphax2 = a2(2)-a2(1);
alphay1 = b1(2)-b1(1);
alphay2 = b2(2)-b2(1);

betaxt = alphax1/alphax2;
betayt = alphay1/alphay2;
vx = false;
vy = false;

if (betaxt > 1)
    betaxt = 1/betaxt;
    vx = true;
end;

if (betayt > 1)
    betayt = 1/betayt;
    vy = true;
end;

if (vx & ~vy)|(~vx & vy)
    warning('The horn is expanding in one dimension and contracting in the other. NOT YET IMPLEMENTED!');
    F = eye(N^2);
    return;
end;

if (betaxt==1)&(betayt==1)
    F = eye(N^2);
    return;
end;

modeindex = MPM_GetRectModeIndexing(N);
L = size(modeindex,1);
ip = 2; in = 1;% indexing

A1 = a1(ip)-a1(in);
A2 = a2(ip)-a2(in);
beta_xt = A1/A2;
if beta_xt > 1
    beta_xt = 1/beta_xt;
    beta_xa = (a2(ip)+a2(in)-2*a1(in))/A1;
else
    beta_xa = (a1(ip)+a1(in)-2*a2(in))/A2;
end;

B1 = b1(ip)-b1(in);
B2 = b2(ip)-b2(in);
beta_yt = B1/B2;
if beta_yt > 1
    beta_yt = 1/beta_yt;
    beta_ya = (b2(ip)+b2(in)-2*b1(in))/B1;
else
    beta_ya = (b1(ip)+b1(in)-2*b2(in))/B2;
end;

rt2 = sqrt(2);
X = ones(L,L); Y = X;
for n=1:L
     for m=1:L
         nx = modeindex(n,1);
         ny = modeindex(n,2);
         mx = modeindex(m,1);
         my = modeindex(m,2);
         if (nx==0)&(mx==0)
             X(n,m) = 1;
         elseif ((nx==0)&(mx>0))
             X(n,m) = rt2*sinc(mx*beta_xt*0.5)*cos(mx*pi*0.5*beta_xa);
         else
             X(n,m) = sinc((nx-mx*beta_xt)/2)*cos(pi/2*(nx-mx*beta_xa))...
                 + sinc((nx+mx*beta_xt)/2)*cos(pi/2*(nx+mx*beta_xa));
         end;
         if (ny==0)&(my==0)
             Y(n,m) = 1;
         elseif ((ny==0)&(my>0))
             Y(n,m) = rt2*sinc(my*beta_yt*0.5)*cos(my*pi*0.5*beta_ya);
         else
             Y(n,m) = sinc((ny-my*beta_yt)/2)*cos(pi/2*(ny-my*beta_ya))...
                 + sinc((ny+my*beta_yt)/2)*cos(pi/2*(ny+my*beta_ya));
         end;
     end;
end;

F = X.*Y;

function FF = OneDimFMatrix(modeindex, a1, a2, xy, forceexact)
% xy is the is the direction, x=1,y=2
L = size(modeindex,1);
ip = 2; in = 1;% indexing
alpha1 = a1(ip)-a1(in);
alpha2 = a2(ip)-a2(in);
beta_xt = alpha1/alpha2;
if beta_xt > 1
    beta_xt = 1/beta_xt;
    beta_xa = (a2(ip)+a2(in)-2*a1(in))/alpha1;
else
    beta_xa = (a1(ip)+a1(in)-2*a2(in))/alpha2;
end;
rt2 = sqrt(2);
FF = zeros(L,L);
for n=1:L
     for m=1:L
         nx = modeindex(n,xy);
         mx = modeindex(m,xy);
         if (nx==0)&(mx==0)
             FF(n,m) = 1;
         elseif ((nx==0)&(mx>0))
             FF(n,m) = rt2*sinc(mx*beta_xt*0.5)*cos(mx*pi*0.5*beta_xa);
         else
             FF(n,m) = sinc((nx-mx*beta_xt)/2)*cos(pi/2*(nx-mx*beta_xa))...
                 + sinc((nx+mx*beta_xt)/2)*cos(pi/2*(nx+mx*beta_xa));
         end;
     end;
end;

