%MPM_RECbaffledradz : Modal radiation impedance matrix
%
% [Z, t] = MPM_RECbaffledradz(k, rho, c, a, b, n, m, usequadrad)
% 
% Calculates the modal radiation impedance for the 
% end of a rectangular duct terminated in an infinite baffle
% by numerical interation.
% Calculates ONLY symmetrical modes.
%
% Input parameters:
% k : wavenumber
% rho : density of medium
% c : sound speed in medium
% a, b : half width and half height of the duct
% n, m : mode numbers n(1) = n_x, n(2) = n_y etc.
% usequadrad : (optional) use quad_rad integration algorithm by A. Asheim
%
% Output parameters:
% Z : modal radiation impedance
% t : calculation time
%
%  --------------------------------------------------- -------------------------------------------
%    This file is part of the Modal Propagation Method (MPM) Toolbox by Bjrn Kolbrek.                       
%                                                                                               
%    The MPM Toolbox is free software: you can redistribute it and/or modify       
%    it under the terms of the GNU General Public License as published by the Free Software     
%    Foundation, either version 3 of the License, or (at your option) any later version.        
%                                                                                                
%    The MPM Toolbox is distributed in the hope that it will be useful,       
%    but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS  
%    FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.             
%                                                                                                
%    You should have received a copy of the GNU General Public License along with the           
%    MPM Toolbox. If not, see <http://www.gnu.org/licenses/>.                 
%  --------------------------------------------------- -------------------------------------------
%

function [Z, t] = MPM_RECbaffledradz(k, rho, c, a, b, n, m, usequadrad)
%Zmat = zeros(maxmodes, maxmodes, length(k));
if nargin < 8
    usequadrad = true;
end;
S = 4*a*b;
nx = n(1);
ny = n(2);
mx = m(1);
my = m(2);

Z = k;
t = 0*k;
f = ffunc(nx,ny,mx,my);
Z3 = (-k.*a.*log(2*k*a)+1.5*k*a).*f;

mode = 0;
testcnt = 0;

n1 = 10;
n2 = 15;

if usequadrad
    t01 = tic;
    Z = conj(-1i*1*k(end)/(2*pi*S).*quad_rad(k(end), @(x,y)ffunction(x,y,n,m,a,b),2*a,2*b,n1,n2));
    tref = toc(t01);
end;

for ii=1:length(k)
    t01 = tic;
    kk = k(ii);  
    if mode==0
        Z(ii) = 1i*rho*c/(2*pi*S)*(Z3(ii) + BaffleradzI1I2(kk, a, b, nx,ny,mx,my, f));
        t(ii) = toc(t01);
        if (ii>2)&&(kk*sqrt(a*b)>2)&&usequadrad
            if t(ii) > 1.5*tref%t(ii-2) % is the calculation time increasing fast?
                mode = 1;
                testcnt = 0;
            end
        end;
    elseif mode==1
        if testcnt == 0 %time for testing?
            % checking the accuracy of quad_rad
            Z(ii) = 1i*rho*c/(2*pi*S)*(Z3(ii) + BaffleradzI1I2(kk, a, b, nx,ny,mx,my, f));
            Zt = conj(-1i*kk*c*rho/(2*pi*S^2).*quad_rad(kk, @(x,y)ffunction(x,y,n,m,a,b),2*a,2*b,n1,n2));
            error = abs((Zt-Z(ii))/Z(ii));
            if error < 0.1
                n1 = 10;
                n2 = 25;
            end;
            %disp(sprintf('checking... k = %.2f, e = %e', kk,error));
            if error < 1e-3 
                mode = 2; % acceptable accuracy, continue with quadrad
                disp(sprintf('transfering to quad_rad at k = %.2f, mode (%d,%d),(%d,%d)', kk,nx,ny,mx,my));
            else
                testcnt = 0;%floor(max(min(log10(error),0),5));% not accurate enough, wait a few frequencies and try again0
            end;
        else
            testcnt = testcnt-1; %wait...
            Z(ii) = 1i*rho*c/(2*pi*S)*(Z3(ii) + BaffleradzI1I2(kk, a, b, nx,ny,mx,my, f));
        end;        
        
    elseif mode==2
%         Zt = 1i*rho*c/(2*pi*S)*(Z3(ii) + BaffleradzI1I2(kk, a, b, nx,ny,mx,my, f));
        Z(ii) = conj(-1i*kk*c*rho/(2*pi*S^2).*quad_rad(kk, @(x,y)ffunction(x,y,n,m,a,b),2*a,2*b,n1,n2));
%         error = abs((Zt-Z(ii))/Z(ii));
%         disp(sprintf('checking mode 2... k = %.2f, e = %e', kk,error));
    end;
    t(ii) = toc(t01);
end;


function Z = BaffleradzI1I2(k, a, b, nx,ny,mx,my, f)
    eps = 10.^(-log(2.5*k*min(a,b))-1);
    eps = max(min(0.01,eps),1e-6);
    k2a = 2*k*a;
    k2b = 2*k*b;
    %disp(sprintf('Symmetric radiation Z for mode (%d,%d),(%d,%d) at k=%f\n',nx,ny,mx,my,kk));
    Z1 = dblquad(@(x,y) Integrand1(x,y,k2a,k2b,nx,ny,mx,my,f), 0,k2a,0,k2b, eps, @myquad);%
    Z2 = quadl(@(x) Integrand2(x,k2a,k2b,nx,ny,mx,my,f),0,k2a,eps); 
    Z = Z1+Z2;

function f = ffunc(nx, ny, mx, my)
% sinc(x) = sin(pi*x)/(pi*x), so skip multiplying by pi.
f = Nnx(nx)*Nnx(mx)*Nnx(ny)*Nnx(my)*(sinc((nx+mx)) + sinc((nx-mx))) * (sinc((ny+my)) + sinc((ny-my)));

function N = Nnx(nx)
if (nx==0)
    N = 1;
else
    N = sqrt(2);
end;

function G = Gfunc(nx, mx, ksi, a, divksi)
if nargin < 5
    divksi = true;
end;
if (ksi==1)&(divksi)
    ksi = 0.999999999;
end;
if (nx==0)&(mx==0)
    G = 2*(2*a-ksi); 
    if divksi
        G = G./(1-ksi);
    end;
else
    % sinc(x) = sin(pi*x)/(pi*x), so skip multiplying by pi.
    sa1 = (nx+mx)*(1-ksi/(2*a));
    ca1 = (nx-mx)*pi*ksi/(2*a);
    sa2 = (nx-mx)*(1-ksi/(2*a));
    ca2 = (nx+mx)*pi*ksi/(2*a);
    G = (2*a-ksi).*( sinc(sa1).*cos(ca1) + sinc(sa2).*cos(ca2));
    if divksi
        G = G ./ (1-ksi);
    end;
    G = Nnx(nx)*Nnx(mx).*G;
end;

function I1 = Integrand1(x,y,ka,kb,nx,ny,mx,my,f)
onem1 = (1-x/ka);
onem2 = (1-y/kb);
rt = sqrt(x.^2+y.^2);
Gf =Gfunc(nx,mx,x/ka,0.5).*Gfunc(ny,my,y/kb,0.5); 
%Gf = Gf./((1-x./(ka)).*(1-y./(kb)));
I1 = onem1.*onem2./rt.*(exp(-1i*rt).*Gf-f);

function I2 = Integrand2(x,ka,kb,nx,ny,mx,my,f)
rt = sqrt(x.^2+(kb).^2);
I2 = (1-x./ka).*(log(kb+rt)+ (x-rt)/kb).*f;


function q = myquad(f,a,b,tol,trace,varargin)
q = quadgk(@(x)f(x,varargin{:}),a,b,'RelTol',tol);

function f = ffunction(x,y,n,m,a,b)
nx = n(1);
ny = n(2);
mx = m(1);
my = m(2);
f =Gfunc(nx,mx,x,a, false).*Gfunc(ny,my,y,b, false); 
