%MPM_RECgeteigenfunctions : calculates the egeinfunctins for a rectangular duct. 
%
% [psi, coords] = MPM_RECgeteigenfunctions(Nmodes, throatdims, Nx, Ny, geomtype)
% 
% Calculates the eigenfunctions for a rectangular duct of dimensions given 
% in throatdims at the coordinates calculated based on Nx and Ny.
%
% Input parameters: 
% Nmodes: number of modes in each direction
% dims: dimensions of the duct, in the same format as ordinary
%      horn coordinates (column 1 is z)
% Nx, Ny : number of sample points in each direction
% geomtype : determines the geometry of the horn:
%    0 : axisymmetric (not supported in this function, use MPM_ASgeteigenfunctions)
%    1 : quarter symmetric rectangular
%    2 : asymmetric rectangular
%    (no other geometries supported yet)
%
% Output parameters:
% psi : the eigenfunction matrix. Each column is a mode combination,
%     each row is a point on the surface, given in coords. 
%     The matrix is (Nx * Ny) rows and (Nmodes^2) columns.
% coords : list of the coordinates used
%
%  ----------------------------------------------------------------------------------------------
%    This file is part of the Modal Propagation Method (MPM) Toolbox by Bjrn Kolbrek.                       
%                                                                                               
%    The MPM Toolbox is free software: you can redistribute it and/or modify       
%    it under the terms of the GNU General Public License as published by the Free Software     
%    Foundation, either version 3 of the License, or (at your option) any later version.        
%                                                                                                
%    The MPM Toolbox is distributed in the hope that it will be useful,       
%    but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS  
%    FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.             
%                                                                                                
%    You should have received a copy of the GNU General Public License along with the           
%    MPM Toolbox. If not, see <http://www.gnu.org/licenses/>.                 
%  ----------------------------------------------------------------------------------------------
%
function [psi, coords] = MPM_RECgeteigenfunctions(Nmodes, dims, Nx, Ny, geomtype, thx, thy)
modeindex = MPM_GetRectModeIndexing(Nmodes);
if geomtype==1
    % get the x and y coordinates at the center of rectangular elements
    if nargin < 6
        thx = ([-Nx/2:Nx/2-1]+0.5)*2*dims(end,2)/Nx;
        thy = ([-Ny/2:Ny/2-1]+0.5)*2*dims(end,3)/Ny;
    end;
%     thx = linspace(-dims(1,2),dims(1,2),Nx);
%     thy = linspace(-dims(1,3),dims(1,3),Ny);
    
    [X,Y] = meshgrid(thx,thy);
    coords = [reshape(X,Nx*Ny,1),reshape(Y,Nx*Ny,1)];% zeros(nx*ny,1)];
    % eigenfunctions, x
    ax = coords(:,1); % x values
    ax = ax(:,ones(1,Nmodes^2));
    modenx = modeindex(:,1)';
    modenx = modenx(ones(size(ax,1),1),:);
    phinx = sqrt(2)*cos(modenx.*ax*pi/dims(1,2));
    m0 = find(modenx==0);
    phinx(m0) = 1;
    % eigenfunctions, y
    ay = coords(:,2); % y values
    ay = ay(:,ones(1,Nmodes^2));
    modeny = modeindex(:,2)';
    modeny = modeny(ones(size(ay,1),1),:);
    sigmay = sqrt(2)*cos(modeny.*ay*pi/dims(1,3));
    m0 = find(modeny==0);
    sigmay(m0) = 1;
elseif geomtype==2
    %thx = ([0:Nt-1]+0.5)*throatdims(1,2)/Nt;% more to do here!!!!!
    aneg = dims(end,2);
    apos = dims(end,3);
    bneg = dims(end,4);
    bpos = dims(end,5);    
    if nargin < 6
        thx = linspace(aneg,apos,Nx);
        thy = linspace(bneg,bpos,Ny);
    end;
    [X,Y] = meshgrid(thx,thy);
    coords = [reshape(X,Nx*Ny,1),reshape(Y,Nx*Ny,1)];% zeros(nx*ny,1)];
    % eigenfunctions, x
    ax = coords(:,1); % x values
    ax = ax(:,ones(1,Nmodes^2));
    modenx = modeindex(:,1)';
    modenx = modenx(ones(size(ax,1),1),:);
%     phinx = sqrt(2)*cos(modenx.*ax*pi/dims(1,2));
    phinx = sqrt(2)*cos(modenx.*(ax-aneg)*pi/(apos-aneg));
    m0 = find(modenx==0);
    phinx(m0) = 1;
    % eigenfunctions, y
    ay = coords(:,2); % y values
    ay = ay(:,ones(1,Nmodes^2));
    modeny = modeindex(:,2)';
    modeny = modeny(ones(size(ay,1),1),:);
    sigmay = sqrt(2)*cos(modeny.*(ay-bneg)*pi/(bpos-bneg));
%     sigmay = sqrt(2)*cos(modeny.*ay*pi/dims(1,3));
    m0 = find(modeny==0);
    sigmay(m0) = 1;    
end;
psi = phinx.*sigmay;