%MPM_RECmakefmat : Make scattering matrix F
% 
% F = MPM_RECmakefmat(N,R1,R2,bz);
% 
% Calculates the scattering matrix F, used to propagate modes across
% a discontinuity (rectangular case).
% 
% Input parameters: 
% N : number of modes in each direction
% a1 : half x-width of tube 1
% a2 : half x-width of tube 2
% b1 : half y-height of tube 1
% b2 : half y-height of tube 2
% forceexact : set to true to avoid approximation for small a1/a2 (not
% implemented)
%
%  --------------------------------------------------- -------------------------------------------
%    This file is part of the Modal Propagation Method (MPM) Toolbox by Bjrn Kolbrek.                       
%                                                                                               
%    The MPM Toolbox is free software: you can redistribute it and/or modify       
%    it under the terms of the GNU General Public License as published by the Free Software     
%    Foundation, either version 3 of the License, or (at your option) any later version.        
%                                                                                                
%    The MPM Toolbox is distributed in the hope that it will be useful,       
%    but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS  
%    FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.             
%                                                                                                
%    You should have received a copy of the GNU General Public License along with the           
%    MPM Toolbox. If not, see <http://www.gnu.org/licenses/>.                 
%  --------------------------------------------------- -------------------------------------------
%

function F = MPM_RECmakefmat(N,a1,a2,b1,b2, forceexact);
if nargin<6 
    forceexact = false;
end;

betax = a1/a2;
betay = b1/b2;
bx=betax;
by=betay;
unitx = false;
unity = false;
vx = false;
vy = false;

if (betax > 1)
    betax = 1/betax;
    vx = true;
elseif (betax==1)
    X = eye(N^2, N^2);
    unitx = true;
end;

if (betay > 1)
    betay = 1/betay;
    vy = true;
elseif (betay==1)
    Y = eye(N^2, N^2);
    unity = true;
end;

if (unitx && unity)
    F = eye(N^2, N^2);
    return;
end;

if ((bx>1) & (by<1))|((bx<1) & (by>1))
    warning('The horn is expanding in one dimension and contracting in the other.');
    % expand first in x
    Fx = MPM_RECmakefmat(N, a1, a2, b1, b1, forceexact);

    % then expand in y
    Fy = MPM_RECmakefmat(N, a2, a2, b1, b2, forceexact);
%     if vy
%         Fy = inv(Fy);
%     end;
    if vx 
        F = Fx\Fy;        
    elseif vy
        F = Fx/Fy;
    else
        F = Fx*Fy; %should not come here...
    end;
    return;
end;
X = zeros(N^2, N^2);
Y = X;
rt2 = sqrt(2);

modeindex = MPM_GetRectModeIndexing(N);
L = size(modeindex,1);

for n=1:L
     for m=1:L
         nx = modeindex(n,1);
         ny = modeindex(n,2);
         mx = modeindex(m,1);
         my = modeindex(m,2);
         if (nx==0)&(mx==0)
             X(n,m) = 1;
         elseif ((nx==0)&(mx>0))
             X(n,m) = rt2*sinc(mx*betax);
         else
             X(n,m) = 2*sinc((mx*betax-nx))*mx*betax/(mx*betax+nx);
         end;
         if (ny==0)&(my==0)
             Y(n,m) = 1;
         elseif ((ny==0)&(my>0))
             Y(n,m) = rt2*sinc(my*betay);
         else
             Y(n,m) = 2*sinc((my*betay-ny))*my*betay/(my*betay+ny);
         end;
     end;
end;

if unitx 
    X = eye(N^2);
end;
if unity
    Y = eye(N^2);
end;

F = X.*Y;

