%MPM_RECmodalradiatedpressure : modal radiation from a rectangular aperture
%
% prext = MPM_RECmodalradiatedpressure(k, a, b, v0, Pe, rho, c)
% 
% Calculates the radiated pressure in front of a baffled 
% rectangular aperture with a given modal velocity distribution.
% The radiator is assumed to be in the z = 0 plane.
% Total number of modes included depends on the size of v0.
%
% Input parameters:
% k : wavenumber
% a, b : x and y dimensions
% v0 : the modal velocity amplitudes 
% Pe : exterior point, (x,y,z). 
% rho : density of the medium
% c : sound speed in medium
%
%  --------------------------------------------------- -------------------------------------------
%    This file is part of the Modal Propagation Method (MPM) Toolbox by Bjrn Kolbrek.                       
%                                                                                               
%    The MPM Toolbox is free software: you can redistribute it and/or modify       
%    it under the terms of the GNU General Public License as published by the Free Software     
%    Foundation, either version 3 of the License, or (at your option) any later version.        
%                                                                                                
%    The MPM Toolbox is distributed in the hope that it will be useful,       
%    but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS  
%    FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.             
%                                                                                                
%    You should have received a copy of the GNU General Public License along with the           
%    MPM Toolbox. If not, see <http://www.gnu.org/licenses/>.                 
%  --------------------------------------------------- -------------------------------------------
%
function prext = MPM_RECmodalradiatedpressure(k, a, b, v0, Pe, rho, c)
Nmodes = sqrt(length(v0(:,1)));
modeindex = MPM_GetRectModeIndexing(Nmodes);
Np = size(Pe,1);
prext = zeros(Np, length(k));
%avoid very small numbers, that may give non-zero values for thetax and
%thetay even if the angle really is zero.
iv = find(abs(Pe) < 1e-6);
Pe(iv) = 0;
for ii = 1:Np
    pe = Pe(ii,:);
    R = norm(pe);    
    thetax = atan2(pe(1),pe(3));%y,x
    thetay = atan2(pe(2),pe(3));
    sx = k*a*sin(thetax);
    sy = k*b*sin(thetay);
    ModalSum = 0;
    for nm = 1:Nmodes^2
       AnBm = v0(nm,:);
       n = modeindex(nm,1);
       m = modeindex(nm,2);
       if n>0
           AnBm = AnBm*sqrt(2);
       end;
       if m>0
           AnBm = AnBm*sqrt(2);
       end;
       Gn = fGn(sx,n);
       Gm = fGn(sy,m);
       ModalSum = ModalSum + AnBm.*Gn.*Gm;
    end;
    pf = 1i*rho*c*(4*a*b)/(2*pi*R)*exp(-1i*k*R).*k;
    prext(ii,:) = pf.*ModalSum;
end;

function g = fGn(x,n)
if n==0
    g = sinc(x/pi);
else
    g = (-1)^n*(x.*sin(x))./(x.^2-(n*pi)^2);
end;