%MPM_getfieldpointpressures : Calculate fiueld point pressures
%
% prext = MPM_getfieldpointpressures(k, Umouth, pe, DimM, Nr, bz, rho, c, geomtype)
% 
% Calculates the sound pressure of a horn in the free field, given the mouth velocity
% and the field point coordinates. The mouth is divided into Nr concentric
% rings, and the particle velocity of each ring is determined. Numerical
% integration is performed over the surface using the Rayleigh integral, to
% obtain the pressure in the free field. 
%
% Input parameters: 
% k : wavenumber (vector)
% Umouth : modal mouth velocity
% pe : field point coordinate list. One point per row. Columns are (z,r) or
%      (z, x, y).
% DimM : mouth dimensions: radius for axisymmetric horns, x,y for
%      rectangular horns.
% Nr : mouth radius resolution: the number of points in the radial direction.
%      Determines partly the resolution of the calculated field point
%      pressure. 
% bzN : zeros of Bessel function J1 (for geomtype 0), number of modes for
%      rectangular geometries.
% rho : density of the medium
% c : sound speed in medium
% geomtype : (optional) determines the geometry of the horn:
%    0 : axisymmetric (default)
%    1 : quarter symmetric rectangular
%    2 : asymmetric rectangular
%    (no other geometries supported yet)
%
%  --------------------------------------------------- -------------------------------------------
%    This file is part of the Modal Propagation Method (MPM) Toolbox by Bjrn Kolbrek.                       
%                                                                                               
%    The MPM Toolbox is free software: you can redistribute it and/or modify       
%    it under the terms of the GNU General Public License as published by the Free Software     
%    Foundation, either version 3 of the License, or (at your option) any later version.        
%                                                                                                
%    The MPM Toolbox is distributed in the hope that it will be useful,       
%    but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS  
%    FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.             
%                                                                                                
%    You should have received a copy of the GNU General Public License along with the           
%    MPM Toolbox. If not, see <http://www.gnu.org/licenses/>.                 
%  --------------------------------------------------- -------------------------------------------
%

function prext = MPM_getfieldpointpressures(k, Umouth, pe, DimM, Nr, bzN, rho, c, geomtype)
if nargin<8 
    geomtype = 0;
end;

prext = [];

if geomtype==0
    r = (0:DimM/(Nr-1):DimM);
    rp2=r(2:end);
    rp1=r(1:end-1);
    rp=(rp1+rp2)/2;
    a = max(r);
    phi = MPM_ASgeteigenfunctions(a, rp,  bzN, true);
    Ur = phi*Umouth; %volume velocity as function of radius
    uo = Ur / (a^2*pi); % mouth particle velocity
    
    prext = zeros(length(pe), length(k));
    Np = size(pe);
    for ii = 1:Np(1)
        pext = pe(ii,:);
        pr = MPM_ASrayleighint(k, r, uo, pext, rho, c);
        prext(ii,:) = pr;
    end;
elseif (geomtype==1)|(geomtype==2)
    [phi, coords] = MPM_RECgeteigenfunctions(bzN, DimM, Nr, Nr, geomtype);
    Ur = phi*Umouth; 
    uo = Ur/(DimM(2)*DimM(3)*4);
    Np = size(pe);  
    prext = zeros(Np(1), length(k));
    % make coordinates 3D, add z-axis
    Lc = size(coords, 1);
    Cz = zeros(Lc, 1);
    coords = [coords Cz];
    dx = DimM(2)*2/Nr;
    dy = DimM(3)*2/Nr;
    dS = dx*dy;
    for ii = 1:Np(1)
        
        pext = pe(ii,:);
        pr = RECrayleighint(k, uo, pext, coords, rho, c, dS);
        prext(ii,:) = pr;
    end;
end;

function pr = RECrayleighint(k, uo, pext, coords, rho, c, dS)
pr = zeros(size(k));
pext = pext(ones(size(coords,1),1),:);
% create vector of distances from pext to all coord points on the surface
r = sqrt(sum(((coords-pext).^2)')'); 
% calculate the Rayleigh integral
for ik = 1:length(k)
    expmat = exp(-1j*k(ik)*r)./r;
    integrand = uo(:,ik).*expmat;
    integral = dS*sum(integrand);
    pr(ik) = 1i*k(ik)*rho*c/(2*pi)*integral;
end;







