%MPM_getimpedances : Calculate the impedances through a horn
%
% BigZ = MPM_getimpedances(k, coords, bz, Zend, BigF, rho, c, geomtype, progressreport)
% 
% Calculates the modal impedances at every duct junction in a horn defined
% by the coordinate list coords. 
% The F matrices (BigF) must have been calculated on beforehand.
% The resulting matrix BigZ contains the modal impedances n,m at point iz
% and wavenumber index ik as BigF(n,m,iz,ik)
%
% Input parameters: 
% k : wavenumber (vector)
% coords : horn coordinates
% bz : zeros of Bessel function J1 (axisymmetric horns only)
% Zend : (radiation) impedance at the mouth end of the horn
% BigF : scattering matrix F for all junctions
% rho : density of the medium
% c : sound speed in medium
% geomtype : (optional) determines the geometry of the horn:
%    0 : axisymmetric (default)
%    1 : quarter symmetric rectangular
%    2 : asymmetric rectangular
%    (no other geometries supported yet)
% progressreport : (boolean, optional) prints the current wavenumber and the
%    percentwise progress. Default is off. Calculations are slightly faster
%    with this option turned off.
%
%  --------------------------------------------------- -------------------------------------------
%    This file is part of the Modal Propagation Method (MPM) Toolbox by Bjrn Kolbrek.                       
%                                                                                               
%    The MPM Toolbox is free software: you can redistribute it and/or modify       
%    it under the terms of the GNU General Public License as published by the Free Software     
%    Foundation, either version 3 of the License, or (at your option) any later version.        
%                                                                                                
%    The MPM Toolbox is distributed in the hope that it will be useful,       
%    but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS  
%    FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.             
%                                                                                                
%    You should have received a copy of the GNU General Public License along with the           
%    MPM Toolbox. If not, see <http://www.gnu.org/licenses/>.                 
%  --------------------------------------------------- -------------------------------------------
%
function BigZ = MPM_getimpedances(k, coords, bz, Zend, BigF, rho, c, geomtype, progressreport)
if nargin<8 
    geomtype = 0;
end;
if nargin<9 
    progressreport = false;
end;

if geomtype == 0 
    S = pi*coords(:,2).^2;
    N = size(BigF);
    N = N(1);
    I = eye(N);
    BigZ = zeros(N,N,length(coords), length(k));
    BigZ(:,:,end,:) = Zend;
    for fi = 1:length(k)
        if progressreport
            disp(['Calculating k = ' num2str(k(fi)) ' (' num2str(fi/length(k)*100) '%)']);
        end;
        kn = MPM_ASmakekm(k(fi),coords(end,2),bz,N);
        Z = Zend(:,:,fi);
        % propagate back to throat
        for iz = (size(coords,1)-1):-1:1
            R1 = coords(iz,2);
            R2 = coords(iz+1,2);
            %R2 = duct(iz+1,1);
            %beta = R1/R2;
            L = coords(iz+1,1) - coords(iz,1);
            if (L>0) %propagate along straight duct
                kn = MPM_ASmakekm(k(fi),R1,bz,N);
                D1 = diag(sin(L*kn));
                D2 = diag(1i*sin(L*kn));
                D3 = diag(tan(L*kn));
                Zc = diag(k(fi)*rho*c./(S(iz)*kn));
                D2Zc = D2\Zc;
                iD3Zc =(1i*D3)\Zc;
                invZc = diag((S(iz)*kn)./(k(fi)*rho*c)); 
                %Z = (Z + 1i*D3*Zc)/(1i*D3*invZc*Z+I);
                %Z = (1i*D3)^-1*Zc - D2^-1*Zc*(Z+(1i*D3)\Zc)^-1*D2^-1*Zc;
                Z = iD3Zc - D2Zc/(Z+iD3Zc) * D2Zc;
            else %propagate across discontinuety
                F = BigF(:,:,iz);
                if R1>R2
                    Z = F\Z/F.'; 
                else
                    Z = F*Z*F.';
                end;
            end;
            BigZ(:,:,iz,fi) = Z; %keep the impedance for velocity forward propagation
        end;
    end;
elseif (geomtype == 1)|(geomtype == 2) % quarter symmetric and asymmetric rectangular
    N2 = size(BigF,1);
    N = sqrt(N2);
    I = eye(N2);
    if (geomtype == 1)
        S = 4*coords(:,2).*coords(:,3);
    else
        S = (coords(:,3)-coords(:,2)).*(coords(:,5)-coords(:,4));
    end;
    modeindex = MPM_GetRectModeIndexing(N);
    BigZ = zeros(N2,N2,size(coords,1), length(k));
    for fi = 1:length(k)
        if progressreport
            disp(['Calculating k = ' num2str(k(fi)) ' (' num2str(fi/length(k)*100) '%)']);
        end;
        %kn = MPM_RECmakekm(k(fi),coords(end,2),coords(end,3),N,modeindex);
               % Zc = diag(k(fi)*rho*c./(S(end)*kn));
        Z = Zend(:,:,fi);
        % propagate back to throat
        for iz = (size(coords,1)-1):-1:1
            if (geomtype == 1)
                alphax1 = coords(iz,2);
                alphax2 = coords(iz+1,2);
                alphay1 = coords(iz,3);
                alphay2 = coords(iz+1,3);      
            else
                a1 = coords(iz,2:3);
                a2 = coords(iz+1,2:3);
                b1 = coords(iz,4:5);
                b2 = coords(iz+1,4:5);
                alphax1 = a1(2)-a1(1);
                alphax2 = a2(2)-a2(1);
                alphay1 = b1(2)-b1(1);
                alphay2 = b2(2)-b2(1);   
            end;
            L = coords(iz+1,1) - coords(iz,1);
            if (L>0) %propagate along straight duct
                if (geomtype == 1)
                    kn = MPM_RECmakekm(k(fi),alphax1,alphay1,N,modeindex);
                else
                    kn = MPM_RECAmakekm(k(fi),a1,b1,N,modeindex);
                end;
                D1 = diag(sin(L*kn));
                D2 = diag(1i*sin(L*kn));
                D3 = diag(tan(L*kn));
                Zc = diag(k(fi)*rho*c./(S(iz)*kn));
                D2Zc = D2\Zc;
                iD3Zc =(1i*D3)\Zc;
                %invZc = diag((S(iz)*kn)./(k(fi)*rho*c)); 
                %Z = (Z + 1i*D3*Zc)/(1i*D3*invZc*Z+I);
                %Z = (1i*D3)^-1*Zc - D2^-1*Zc*(Z+(1i*D3)\Zc)^-1*D2^-1*Zc;
                Z = iD3Zc - D2Zc/(Z+iD3Zc) * D2Zc;
                
                %D3 = diag(tan(L*kn));
                %Zc = diag(k(fi)*rho*c./(S(iz)*kn));
                %invZc = diag((S(iz)*kn)./(k(fi)*rho*c));
                %Z = (Z + 1i*D3*Zc)/(1i*D3*invZc*Z+I);
                %%Z = (1i*D3)^-1*Zc - D2^-1*Zc*(Z+(1i*D3)\Zc)^-1*D2^-1*Zc;
                %%Z = D3\Zc - D2\(Z+D3\Zc)\D2\I;
            else %propagate across discontinuety
                F = BigF(:,:,iz);
                if (alphax1>alphax2) & (alphay1>alphay2)% contraction in both directions
                    Z = F\Z/F.'; 
                else
                    Z = F*Z*F.';
                end;
            end;
            BigZ(:,:,iz,fi) = Z; %keep the impedance for velocity forward propagation
        end;
    end;
end;