%MPM_getmouthvolvel : Calculate mouth modal volume velocity
%
% Umouth = MPM_getmouthvolvel(k, coords, U0, bz, BigZ, BigF, rho, c, geomtype)
% 
% Calculates the modal volume velocity at the mouth of a horn defined
% by the coordinate list coords, given a modal throat velocity U0. 
% The F and Z matrices (BigF, BigZ) must have been calculated on beforehand.
%
% Input parameters: 
% k : wavenumber (vector)
% coords : horn coordinates
% U0 : throat modal velocity
% bz : zeros of Bessel function J1 (for geomtype = 1)
% BigZ : impedance matrices for all points in the horn, and all wavenumbers
% BigF : scattering matrix F for all junctions
% rho : density of the medium
% c : sound speed in medium
% geomtype : (optional) determines the geometry of the horn:
%    0 : axisymmetric (default)
%    1 : quarter symmetric rectangular
%    2 : asymmetric rectangular
%    (no other geometries supported yet)
%
%  --------------------------------------------------- -------------------------------------------
%    This file is part of the Modal Propagation Method (MPM) Toolbox by Bjrn Kolbrek.                       
%                                                                                               
%    The MPM Toolbox is free software: you can redistribute it and/or modify       
%    it under the terms of the GNU General Public License as published by the Free Software     
%    Foundation, either version 3 of the License, or (at your option) any later version.        
%                                                                                                
%    The MPM Toolbox is distributed in the hope that it will be useful,       
%    but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS  
%    FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.             
%                                                                                                
%    You should have received a copy of the GNU General Public License along with the           
%    MPM Toolbox. If not, see <http://www.gnu.org/licenses/>.                 
%  --------------------------------------------------- -------------------------------------------
%

function Umouth = MPM_getmouthvolvel(k, coords, U0, bz, BigZ, BigF, rho, c, geomtype)
if nargin<8 
    geomtype = 0;
end;

Umouth = [];
SS = size(U0);

if geomtype==0
    S = pi*coords(:,2).^2;
    N = size(BigF,1);
    for ki = 1:length(k)
        if SS(2)>1
             U = U0(:,ki);
        else
             U = U0;
        end;
        Sm = S(end);
        for iz = 1:length(coords)-1 
            R1 = coords(iz,2);
            R2 = coords(iz+1,2);
            L = coords(iz+1,1) - coords(iz,1);
            if (L>0) 
                Z0 = BigZ(:,:,iz,ki);
                kn = MPM_ASmakekm(k(ki),R1,bz,N);  
                D2 = diag(1i*sin(L*kn));
                E = diag(exp(-1i*L*kn));
                Zc = diag(k(ki)*rho*c./(S(iz)*kn));
                invZc = diag((S(iz)*kn)./(k(ki)*rho*c));
                U = (-D2*invZc*(Z0-Zc)+E)*U;
            else
                F = BigF(:,:,iz);
                if R1>R2
                    U = (F.')\U;
                else
                    U = F.'*U;
                end;
            end;
        end;
        Umouth = [Umouth U];
    end;
elseif (geomtype == 1)|(geomtype == 2) % quarter symmetric and asymmetric rectangular
    N2 = size(BigF,1);
    N = sqrt(N2);
    if (geomtype == 1)
        S = 4*coords(:,2).*coords(:,3);
    else
        S = (coords(:,3)-coords(:,2)).*(coords(:,5)-coords(:,4));
    end;
    modeindex = MPM_GetRectModeIndexing(N);
    for ki = 1:length(k)
        if SS(2)>1
             U = U0(:,ki);
        else
             U = U0;
        end;  
        Sm = S(end);
        
        for iz = 1:length(coords)-1 
            if (geomtype == 1)
                alphax1 = coords(iz,2);
                alphax2 = coords(iz+1,2);
                alphay1 = coords(iz,3);
                alphay2 = coords(iz+1,3);      
            else
                a1 = coords(iz,2:3);
                a2 = coords(iz+1,2:3);
                b1 = coords(iz,4:5);
                b2 = coords(iz+1,4:5);
                alphax1 = a1(2)-a1(1);
                alphax2 = a2(2)-a2(1);
                alphay1 = b1(2)-b1(1);
                alphay2 = b2(2)-b2(1);   
            end;
            L = coords(iz+1,1) - coords(iz,1);
            if (L>0) 
                Z0 = BigZ(:,:,iz,ki);
                if (geomtype == 1)
                    kn = MPM_RECmakekm(k(ki),alphax1,alphay1,N,modeindex);
                else
                    kn = MPM_RECAmakekm(k(ki),a1,b1,N,modeindex);
                end;
%                 kn = MPM_ASmakekm(k(ki),R1,bz,N);  
                D2 = diag(1i*sin(L*kn));
                E = diag(exp(-1i*L*kn));
                Zc = diag(k(ki)*rho*c./(S(iz)*kn));
                invZc = diag((S(iz)*kn)./(k(ki)*rho*c));
                U = (-D2*invZc*(Z0-Zc)+E)*U;
            else
                F = BigF(:,:,iz);
                if (alphax1>alphax2) & (alphay1>alphay2)
                    U = (F.')\U;
                else
                    U = F.'*U;
                end;
            end;
        end;
        Umouth = [Umouth U]; 
 
    end;

end;    