%MPM_hornspeaker class to hold driver and front/rear chamber parameters
%   
% This class contains the electromechanical parameters for a loudspeaker
% driver, and functions to calculate output volume velocity and system
% electrical impedance, given the radiation load.
% It also contains the dimensions of the front and rear chambers, and
% electrical input voltage and source resistance. 
% The front chamber values are not used directly in the calculations, the
% class mainly serves as a place holder for them.
%
% properties with default values:
%     Sd = 350e-4;    % m^2
%     Re = 6.0;       % ohms
%     Le = 1.0e-3;    % H
%     BL = 18;        % Tm
%     Cms = 4.0e-4;   % m/N
%     Rms = 4.0;      % Ns/m
%     Mmd = 20.0e-3;  % kg
% 
%     % Front and rear chamber parameters
%     Vrc = 14e-3;    % m^3
%     Lrc = 16e-2;    % m
%     Vtc = 900e-6;   % m^3
%     Atc = 350e-4;   % m^2
% 
%     % Generator parameters
%     Eg = 2.83;      % V
%     Rg = 0;         % ohms
% 
%     %other parameters
%     c = 344;        % m/s
%     rho = 1.205;    % kg/m^3
%     f;              % frequency vector
%     Zrad;           % radiation load vector
%     N;              % number of modes (when generating U0)
%
% methods: 
%    function U0 = getvolumevelocity : calculates the modal volume
%       velocity into Zrad, using N modes. f, Zrad and N must be set before
%       calling this function.
%    function Ze = getelZ(obj) : calculates the electrical input impedance
%       of the system.  f and Zrad must be set before calling this function.
%
%  --------------------------------------------------- -------------------------------------------
%    This file is part of the Modal Propagation Method (MPM) Toolbox by Bjrn Kolbrek.                       
%                                                                                               
%    The MPM Toolbox is free software: you can redistribute it and/or modify       
%    it under the terms of the GNU General Public License as published by the Free Software     
%    Foundation, either version 3 of the License, or (at your option) any later version.        
%                                                                                                
%    The MPM Toolbox is distributed in the hope that it will be useful,       
%    but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS  
%    FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.             
%                                                                                                
%    You should have received a copy of the GNU General Public License along with the           
%    MPM Toolbox. If not, see <http://www.gnu.org/licenses/>.                 
%  --------------------------------------------------- -------------------------------------------
%
classdef MPM_hornspeaker
    properties
        Sd = 350e-4;    % m^2
        Re = 6.0;       % ohms
        Le = 1.0e-3;    % H
        BL = 18;        % Tm
        Cms = 4.0e-4;   % m/N
        Rms = 4.0;      % Ns/m
        Mmd = 20.0e-3;  % kg

        % Front and rear chamber parameters
        Vrc = 14e-3;    % m^3
        Lrc = 16e-2;    % m
        Vtc = 900e-6;   % m^3
        Atc = 350e-4;   % m^2

        % Generator parameters
        Eg = 2.83;      % V
        Rg = 0;         % ohms
        
        %other parameters
        c = 344;        % m/s
        rho = 1.205;    % kg/m^3
        f;              % frequency vector
        Zrad;           % radiation load vector
        N;              % number of modes (when generating U0)
    end
    methods
        function obj = MPM_hornspeaker(Sd, Re, Le, BL, Cms, Rms, Mmd, Vrc, Lrc, Vtc, Atc, Eg, Rg)
            if nargin > 0
                obj.Sd = Sd; %m^2
                obj.Re = Re; % ohms
                obj.Le = Le; % H
                obj.BL = BL; % Tm
                obj.Cms = Cms; % m/N
                obj.Rms = Rms; %Ns/m
                obj.Mmd = Mmd; %kg
            end;
            if (nargin > 7)
                % Front and rear chamber parameters
                obj.Vrc = Vrc; %m^3
                obj.Lrc = Lrc; %m
                obj.Vtc = Vtc; %m^3
                obj.Atc = Atc; % m^2
            end;
            if nargin >11 
                % Generator parameters
                obj.Eg = Eg;
                obj.Rg = Rg;
            end;
        end
        function U0 = getvolumevelocity(obj)
            Nf = length(obj.f);
            k = 2*pi*obj.f/obj.c;
            w = 2*pi*obj.f;
            % Calculate driver acoustical source impedance

            Zee = obj.Rg+obj.Re+1i*w*obj.Le;
            Zme = obj.BL^2./Zee;
            Zm = obj.Rms + 1i*w*obj.Mmd + 1./(1i*w*obj.Cms);
            Zmt = Zme + Zm;
            Zas = Zmt*1/obj.Sd^2;
            Ps = obj.Eg*obj.BL/obj.Sd./Zee;
            if (obj.Lrc*obj.Vrc)>0
                Zab = -1i*obj.rho*obj.c/(obj.Vrc/obj.Lrc)*cot(k*obj.Lrc);
            else
                Zab = 0;
            end;
            U0 = zeros(obj.N,Nf); %plane wave at throat
            U0(1,:) = (Ps./(Zas+Zab+obj.Zrad.')).';
        end
        
        function Ze = getelZ(obj)
            Nf = length(obj.f);
            k = 2*pi*obj.f/obj.c;
            w = 2*pi*obj.f;
            
            if (obj.Lrc*obj.Vrc)>0
                Zab = -1i*obj.rho*obj.c/(obj.Vrc/obj.Lrc)*cot(k*obj.Lrc);
            else
                Zab = 0;
            end;
            Zat = Zab + obj.Zrad.';
            Zma = Zat * obj.Sd^2;
            Zmt = Zma + obj.Rms + 1i*w*obj.Mmd + 1./(1i*w*obj.Cms);
            Zem = obj.BL^2./Zmt;
            Zee = obj.Rg+obj.Re+1i*w*obj.Le;
            Ze = Zem + Zee;
        end
            
    end
 end

