%MPM_makebigfmat : calculate F matrices throughout a horn
% 
% bigF = MPM_makebigfmat(N, coords, bz, geomtype)
% 
% Calculates the scattering matrices F  for all discontinuities  in the 
% horn. bigF(:,:,i) is the matrix F at position i in the horn.
%  
% Input parameters: 
% N : number of modes (in each direction for rectangular horns)
% coords : horn coordinates
% bz : zeros of Bessel function J1 (axisymmetric horns only)
% geomtype : determines the geometry of the horn:
%    0 : axisymmetric (default)
%    1 : quarter symmetric rectangular
%    2 : asymmetric rectangular
%    (no other geometries supported yet)
%
%  --------------------------------------------------- -------------------------------------------
%    This file is part of the Modal Propagation Method (MPM) Toolbox by Bjrn Kolbrek.                       
%                                                                                               
%    The MPM Toolbox is free software: you can redistribute it and/or modify       
%    it under the terms of the GNU General Public License as published by the Free Software     
%    Foundation, either version 3 of the License, or (at your option) any later version.        
%                                                                                                
%    The MPM Toolbox is distributed in the hope that it will be useful,       
%    but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS  
%    FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.             
%                                                                                                
%    You should have received a copy of the GNU General Public License along with the           
%    MPM Toolbox. If not, see <http://www.gnu.org/licenses/>.                 
%  --------------------------------------------------- -------------------------------------------
%
function bigF = MPM_makebigfmat(N, coords, bz, geomtype)
if nargin<4 
    geomtype = 0;
end;

Lc = size(coords,1);
if geomtype == 0 % axisymmetric horn
    bigF = zeros(N,N,Lc);
    for iz = 1:Lc-1% (length(coords)-1):-1:1
        L = coords(iz+1,1) - coords(iz,1);
        if (L==0) %propagate across discontinuety
            R1 = coords(iz,2);
            R2 = coords(iz+1,2);
            F = MPM_ASmakefmat(N,R1,R2,bz); 
            bigF(:,:,iz)=F;
        end;
    end;
elseif geomtype == 1 % quarter symmetric rectangular
    bigF = zeros(N^2,N^2,Lc);
    for iz = 1:Lc-1% (length(coords)-1):-1:1
        L = coords(iz+1,1) - coords(iz,1);
        if (L==0) %propagate across discontinuety
            a1 = coords(iz,2);
            a2 = coords(iz+1,2);
            b1 = coords(iz,3);
            b2 = coords(iz+1,3);
            F = MPM_RECmakefmat(N,a1,a2,b1,b2); 
            bigF(:,:,iz)=F;
        end;
    end;
elseif geomtype == 2 %asymmetric rectangular
    bigF = zeros(N^2,N^2,Lc);
    for iz = 1:Lc-1% (length(coords)-1):-1:1
        L = coords(iz+1,1) - coords(iz,1);
        if (L==0) %propagate across discontinuety
            a1 = coords(iz,2:3);
            a2 = coords(iz+1,2:3);
            b1 = coords(iz,4:5);
            b2 = coords(iz+1,4:5);
            F = MPM_RECAmakefmat(N,a1,a2,b1,b2,false); 
            bigF(:,:,iz)=F;
        end;
    end;    
    %F = MPM_RECAmakefmat(N,a1,a2,b1,b2, forceexact);
else
    error(['Geometry type ' num2str(geomtype) ' not supported!']);
end;