%MPM_readHornDims : Reads horn dimensions from a text file
%
% [horndims, n, geomtype] = MPM_readHornDims(fname)
% 
% Reads horn dimensions exported from Horn Simulator.
% The file contains a header that determines the geometry,
% and a list of coordinates. 
% The cordinate list is output in horndims.
% The columns are arranged as follows:
% For axisymmetrid horns: 
%   col 1 = length along horn (z)
%   col 2 = radius (r)
% For rectangular, quarter symmetric horns: 
%   col 1 = length along horn (z) 
%   col 2 = width of horn, left (a+)
%   col 3 = width of horn, right (b+)
% For rectangular horns: 
%   col 1 = length along horn (z) 
%   col 2 = width of horn, left (a-)
%   col 3 = width of horn, right (a+)
%   col 4 = height of horn, down (b-)
%   col 5 = height of horn, up (b+)
%   (directions refer to the horn viewed from the mouth)
%
% Input parameters:
% fname : file name of the textfile
%
% Output parameters:
% horndims : the matrix containing the horn dimensions
% n : the number of elements
% geomtype : determines the geometry of the horn:
%    0 : axisymmetric (default)
%    1 : quarter symmetric rectangular
%    2 : asymmetric rectangular
%    (no other geometries supported yet)
%
%  --------------------------------------------------- -------------------------------------------
%    This file is part of the Modal Propagation Method (MPM) Toolbox by Bjrn Kolbrek.                       
%                                                                                               
%    The MPM Toolbox is free software: you can redistribute it and/or modify       
%    it under the terms of the GNU General Public License as published by the Free Software     
%    Foundation, either version 3 of the License, or (at your option) any later version.        
%                                                                                                
%    The MPM Toolbox is distributed in the hope that it will be useful,       
%    but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS  
%    FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.             
%                                                                                                
%    You should have received a copy of the GNU General Public License along with the           
%    MPM Toolbox. If not, see <http://www.gnu.org/licenses/>.                 
%  --------------------------------------------------- -------------------------------------------
%

function [horndims, n, geomtype] = MPM_readHornDims(fname);
% reads horn dimensions exported from HornCAD
fid=fopen(fname, 'rt');   %open the file
if fid==-1
    error('File does not exist');
    return;
end;
horndims = [];
s = fgetl(fid);  %read the first line
s = fgetl(fid);  %read the second line
a = findstr(s, 'Radius');
r = findstr(s, 'Height1');
if length(a)>0
    % circular/axisymmetric horn
    s = fgetl(fid);
    s = fgetl(fid);
    [dims, count] = fscanf(fid, '%e');
    horndims = reshape(dims, 2, count/2)';
    n = count/2;
    geomtype = 0;
elseif length(r)>0
    % rectangular horn
    s = fgetl(fid);
    s = fgetl(fid);
    s = fgetl(fid);
    s = fgetl(fid);
    s = fgetl(fid);
    [dims, count] = fscanf(fid, '%e');
    n = count/5;
    dims = reshape(dims, 5, count/5)';
    dims = dims*1e-2; % file contains dimensions in cm
    % rearrange into a-, a+, b-, b+
    z = dims(:,1);
    bp = dims(:,2);
    bn = -dims(:,3);
    an = -dims(:,4);
    ap = dims(:,5);
    if (max(abs(an+ap)) < 1e-6) & (max(abs(bn+bp)) < 1e-6)
        geomtype = 1;
        horndims = [z ap bp];
    else
        geomtype = 2;
        horndims = [z an ap bn bp];
    end;
end;    
fclose(fid);   %close the file